@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-my $revision = '$Id: Sanitizer.pm,v 1.87 2004/05/07 17:42:12 bre Exp $';
+my $revision = '$Id: Sanitizer.pm,v 1.89 2004/09/02 10:14:15 bre Exp $';
 my $version = 'Anomy 0.0.0 : Sanitizer.pm';
 #
 ##  Copyright (c) 2000-2004 Bjarni R. Einarsson. All rights reserved.
@@ -51,7 +51,7 @@ use bytes;
 BEGIN {
     use Exporter  ();
     use vars      qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION      = do { my @r = (q$Revision: 1.87 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+    $VERSION      = do { my @r = (q$Revision: 1.89 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
     @ISA          = qw(Exporter);
     @EXPORT       = qw( );
     @EXPORT_OK    = qw( );
@@ -71,6 +71,7 @@ my $default_config =
     "feat_log_trace"  => 0,  # Attachment logs: Complete (all trace info).
 	"feat_log_after"  => 0,  # Scratch area sizes - see below. 0 = Off.
     "feat_files"      => 1,  # Enable filename-based policy decisions.
+	"feat_sane_names" => 1,  # Make sure file names match the content-type.
 	"feat_mime_files" => 0,  # Always check the mime-type's default name too.
     "feat_force_name" => 0,  # Force all parts (except text/plain or
                              # text/html parts) to have file names.
@@ -1277,17 +1278,29 @@ sub SanitizeFile
 
 	# Get a list of all possible file names for this attachment
 	my @filenames = map { ($_->{"data"}, $_->{"raw"}) } $part->GetMIMEAttributes('(?i)^(file)?name$');
+	push @filenames, $$fnp;
+    
+    # Make sure the list doesn't repeat itself.
+	my $l = undef;
+     @filenames = grep { ($_ ne $l) && ($l = $_) } sort @filenames;
+    my %filenames = map { $_ => 1 } @filenames;
+
+    # Look even harder...
 	if ($conf->{"feat_mime_files"})
 	{
 	    my @mimetypes = map { ($_->{"data"}, $_->{"raw"}) } $part->GetMIMEAttributes('(?i)^_type$');
 		foreach my $t (@mimetypes)
 		{
 		    my $name = $self->{"filenames"}->{lc($t)};
-			push @filenames, $name if ($name);
+            if ($name && !$filenames{$name})
+            {
+			    push @filenames, $name;
+                $filenames{$name} = 1;
+            }
 		}
 	}
-	push @filenames, $$fnp;
 
+    # And keep looking harder...
 	foreach my $h ("_description", "_id")
 	{
 	    foreach my $v (map { ($_->{"data"}, $_->{"raw"}) } $part->GetMIMEAttributes("(?i)^$h\$"))
@@ -1295,12 +1308,14 @@ sub SanitizeFile
 		    $v = $1 if ($v =~ /^<+(.*?)>+\s*$/);
             # Skip @foo.com type names, they shouldn't match anyway.
             next if ($v =~ /\@\S+\.com$/i);
-			push @filenames, $v if ($v =~ /\./);
+            if (($v =~ /\./) && (!$filenames{$v}))
+            {
+			    push @filenames, $v; 
+                $filenames{$v} = 1;
+            }
 		}
 	}
 
-	my $l = undef;
-	@filenames = grep { ($_ ne $l) && ($l = $_) } sort @filenames;
 	my $filenames = join(', ', @filenames);
 
     # Insert our log into part log.
@@ -1478,8 +1493,12 @@ sub SanitizeFile
                 my $fn = $filename;
                 $fn =~ s/^.*\///g;
 
-	            $conf->{"var_filename"} = $$fnp;
-				$conf->{"var_savedname"} = $fn;
+                # Make info available to the replacement template code.
+	            $conf->{"var_filename"}    = $$fnp;
+				$conf->{"var_savedname"}   = $fn;
+				$conf->{"var_virusname"}   = $self->{"filescan-virusname"};
+                $conf->{"var_summary"}     = $self->{"filescan-summary"};
+                $conf->{"var_description"} = $self->{"filescan-description"};
 
                 # This keeps the file from getting reincluded.
                 while ($part->Read()) { };
@@ -1545,7 +1564,8 @@ sub SanitizeFile
 
 	# Ensure the file name matches the MIME type
 	my $check = undef;
-	if (($check = $self->{"name_type_checks"}->{lc($$typep)}) && 
+	if (($conf->{"feat_sane_names"}) &&
+        ($check = $self->{"name_type_checks"}->{lc($$typep)}) && 
 	    ($$fnp !~ $check))
 	{
         $log->entry("mime-type_mismatch", SLOG_INFO, 
